
// optimized method signatures

vec3    luminance_lut( in vec3 color );

vec3    vibrance( in vec3 color );
vec3    white_balance_srgb( in vec3 color );
vec3    color_adjust( in vec3 color );

float   greyscale_blend( in vec3 color );
vec3    greyscale_tint( in float grey );
vec3    greyscale_adjust( in vec3 color );
float   greyscale_lum( in vec3 color );

float   alpha_adjust( in float alpha );

vec3    final_mix( in vec3 color, in vec3 grey, in float alpha );
vec3    ruby_mix(  in vec3 grey, in float alpha );

// ---------------------------------------------------------------------

uniform lowp sampler2D  u_contrastLut;

vec3
luminance_lut( in vec3 color )
{
    color.r    = texture2D( u_contrastLut, vec2( color.r, .5 ) ).r;
    color.g    = texture2D( u_contrastLut, vec2( color.g, .5 ) ).r;
    color.b    = texture2D( u_contrastLut, vec2( color.b, .5 ) ).r;
    
    return color;
}

// ---------------------------------------------------------------------

uniform lowp float      u_hsvSaturation;
uniform lowp vec3       u_whiteBalanceColorSrgb;

vec3
vibrance( in vec3 color )
{
    vec3 grey                               = vec3( greyscale_lum( color ) );
    mediump vec3 colorDiff                  = color - grey;
    // We add 0.001 here to avoid division by 0 problems when later dividing by colorMagnitude
    mediump float colorMagnitude            = length( colorDiff ) + 0.001;
    mediump float saturatedColorMagnitude   = pow( colorMagnitude, 1.0 / u_hsvSaturation);
    mediump float saturationFactor          = saturatedColorMagnitude / colorMagnitude;
    vec3 newrgb                             = grey + saturationFactor * colorDiff;
    return newrgb;
}

vec3
white_balance_srgb( in vec3 color )
{
    color = color * u_whiteBalanceColorSrgb;
    return color;
}

vec3
color_adjust( in vec3 color )
{
    color = white_balance_srgb( color );
    color = vibrance( color );
    color = clamp( color, 0., 1. );
    return color;
}

// ---------------------------------------------------------------------

uniform lowp vec3       u_greyBlendColor;
uniform lowp vec3       u_greyTintColor;
uniform lowp vec3       u_greyChannelWeights;

float
greyscale_blend( in vec3 color )
{
    return dot( color, u_greyBlendColor );
}

vec3
greyscale_tint( in float grey )
{
    return vec3( grey ) * u_greyTintColor;
}

vec3
greyscale_adjust( in vec3 color )
{
    return greyscale_tint( greyscale_blend( color ) );
}

float
greyscale_lum( in vec3 color )
{
    return dot( color, u_greyChannelWeights );
}

// ---------------------------------------------------------------------

uniform lowp float      u_minAlpha;
uniform lowp float      u_maxAlpha;

float
alpha_adjust( in float alpha )
{
    return mix( u_minAlpha, u_maxAlpha, alpha );
}

// ---------------------------------------------------------------------

vec3
final_mix( in vec3 color, in vec3 grey, in float alpha )
{
    alpha   = alpha_adjust( alpha );
    return mix( color, grey, alpha );
}

vec3
ruby_mix(  in vec3 grey, in float alpha )
{
    alpha           = alpha_adjust( alpha );
    lowp vec3 red   = vec3( 1., 0., 0. );
    return mix( red, grey, alpha );
}
