
precision lowp float;

// ---------------------------------------------------------------------

varying mediump vec2    v_iTexCoord;
varying mediump vec2    v_aTexCoord;

uniform lowp sampler2D  u_image;
uniform lowp sampler2D  u_alphaMask;

// ---------------------------------------------------------------------

vec3    luminance_lut( in vec3 color );

vec3    vibrance( in vec3 color );
vec3    white_balance_srgb( in vec3 color );
vec3    color_adjust( in vec3 color );

float   greyscale_blend( in vec3 color );
vec3    greyscale_tint( in float grey );
vec3    greyscale_adjust( in vec3 color );
float   greyscale_lum( in vec3 color );

float   alpha_adjust( in float alpha );

vec3    final_mix( in vec3 color, in vec3 grey, in float alpha );
vec3    ruby_mix(  in vec3 grey, in float alpha );

// ---------------------------------------------------------------------

vec3 process_norm( in vec3 color, in float alpha );
vec3 process_ruby( in vec3 color, in float alpha );
vec3 process_noadjust( in vec3 color, in float alpha );
vec3 process_noadjust_ruby( in vec3 color, in float alpha );

// ---------------------------------------------------------------------

void main()
{    
    lowp vec3 color      = texture2D( u_image, v_iTexCoord ).rgb;
    lowp float alpha     = texture2D( u_alphaMask,  v_aTexCoord ).a;
    
#ifndef PP_RUBYLITH
    #ifndef PP_DISABLE_ADJUSTMENTS

    // [ ] PP_RUBYLITH
    // [ ] PP_DISABLE_ADJUSTMENTS
    color = process_norm( color, alpha );

    #else

    // [ ] PP_RUBYLITH
    // [x] PP_DISABLE_ADJUSTMENTS
    color = process_noadjust( color, alpha );

    #endif
#else
    #ifndef PP_DISABLE_ADJUSTMENTS
    
    // [x] PP_RUBYLITH
    // [ ] PP_DISABLE_ADJUSTMENTS
    color = process_ruby( color, alpha );

    #else
    
    // [x] PP_RUBYLITH
    // [x] PP_DISABLE_ADJUSTMENTS
    color = process_noadjust_ruby( color, alpha );

    #endif
#endif
    
    gl_FragColor    = vec4( color, 1. );
}

// ---------------------------------------------------------------------

vec3
process_norm( in vec3 color, in float alpha )
{
    // luminance lut
    color = luminance_lut( color );
    
    // blend and tint grey
    lowp vec3 grey  = greyscale_adjust( color );
    
    // adjust saturation and white balance
    color = color_adjust( color );
    
    // mix color and grey
    color = final_mix( color, grey, alpha );
    
    return color;
}

vec3
process_ruby( in vec3 color, in float alpha )
{
    // luminance lut
    color = luminance_lut( color );
    
    // blend and tint grey
    lowp vec3 grey  = greyscale_adjust( color );
    
    // mix ruby
    color = ruby_mix( grey, alpha );
    
    return color;
}

vec3
process_noadjust( in vec3 color, in float alpha )
{
    // blend and tint grey
    lowp vec3 grey  = vec3( greyscale_blend( color ) );
    
    // mix color and grey
    color = final_mix( color, grey, alpha );
    
    return color;
}

vec3
process_noadjust_ruby( in vec3 color, in float alpha )
{
    // blend and tint grey
    lowp vec3 grey  = vec3( greyscale_blend( color ) );
    
    // mix ruby
    color = ruby_mix( grey, alpha );
    
    return color;
}
