
precision lowp float;

// ---------------------------------------------------------------------

varying mediump vec2    v_iTexCoord;
varying mediump vec2    v_aTexCoord;

uniform lowp sampler2D  u_image;
uniform lowp sampler2D  u_alphaMask;

// ---------------------------------------------------------------------

vec3    luminance_lut( in vec3 color );

vec3    vibrance( in vec3 color );
vec3    white_balance_srgb( in vec3 color );
vec3    color_adjust( in vec3 color );

float   greyscale_blend( in vec3 color );
vec3    greyscale_tint( in float grey );
vec3    greyscale_adjust( in vec3 color );
float   greyscale_lum( in vec3 color );

float   alpha_adjust( in float alpha );

vec3    final_mix( in vec3 color, in vec3 grey, in float alpha );
vec3    ruby_mix(  in vec3 grey, in float alpha );

// ---------------------------------------------------------------------

vec3
process_norm( in vec4 rgbl, in float alpha )
{
    // tint grey
    lowp vec3 grey  = greyscale_tint( rgbl.a );
    
    // mix color and grey
    lowp vec3 color = final_mix( rgbl.rgb, grey, alpha );
    
    return color;
}

vec3
process_ruby( in vec4 rgbl, in float alpha )
{
    // blend and tint grey
    lowp vec3 grey  = greyscale_tint( rgbl.a );
    
    // mix ruby
    lowp vec3 color = ruby_mix( grey, alpha );
    
    return color;
}

// ---------------------------------------------------------------------

void main()
{    
    lowp vec4 rgbl      = texture2D( u_image, v_iTexCoord );
    lowp float alpha    = texture2D( u_alphaMask,  v_aTexCoord ).a;

    lowp vec3 color;

#ifndef PP_RUBYLITH
    
    // final mixing
    color = process_norm( rgbl, alpha );

#else
    
    // rubylith mixing
    color = process_ruby( rgbl, alpha );
    
#endif
    
    gl_FragColor    = vec4( color, 1. );

}
