//precision mediump float;
//precision lowp float;

float SRGB_jump     = 0.04045;
float SRGB_jumpinv	= 0.0031308;
float SRGB_loscale	= 12.92;
float SRGB_hiscale	= 1.055;
float SRGB_hibias	= 0.055;
float SRGB_hiexp	= 2.4;

float SRGB_optexp      = 2.2;
float SRGB_optexpinv   = 0.45454545;

mat3 XYZ_M 		= mat3(  	0.4124564,  0.3575761,  0.1804375,
                            0.2126729,  0.7151522,  0.0721750,
                            0.0193339,  0.1191920,  0.9503041 );

mat3 XYZ_Minv 	= mat3( 	 3.2404542, -1.5371385, -0.4985314,
                            -0.9692660,  1.8760108,  0.0415560,
                             0.0556434, -0.2040259,  1.0572252 );

vec3 XYZ_D65	= vec3(      0.95047,    1.0,        1.08883 );

mat3 YIQ_M      = mat3(      0.299,      0.587,      0.144,
                             0.595716,  -0.274453,  -0.321263,
                             0.211456,  -0.522591,   0.311135 );

mat3 YIQ_Minv   = mat3(      1.0,        0.9563,     0.6210,
                             1.0,       -0.2721,    -0.6474,
                             1.0,       -1.1070,     1.7046 );

// --------------------------------------------------------------------------------

vec3    srgb_to_linear_ref( in vec3 srgb );
vec3    linear_to_srgb_ref( in vec3 linear );

vec3    srgb_to_linear_opt( in vec3 srgb );
vec3    linear_to_srgb_opt( in vec3 linear );

vec3    rgb_to_xyz( in vec3 rgb );
vec3    xyz_to_rgb( in vec3 xyz );

vec3    rgb_to_yiq( in vec3 rgb );
vec3    yiq_to_rgb( in vec3 yiq );

vec3    rgb_to_hsv( in vec3 rgb );
vec3    hsv_to_rgb( in vec3 hsv );

float   min_channel( in vec2 v );
float   min_channel( in vec3 v );
float   min_channel( in vec4 v );
float   max_channel( in vec2 v );
float   max_channel( in vec3 v );
float   max_channel( in vec4 v );

float   sum( in vec2 v );
float   sum( in vec3 v );
float   sum( in vec4 v );

float   mean( in vec2 v );
float   mean( in vec3 v );
float   mean( in vec4 v );

vec2    pow( in vec2 v, in float e );
vec3    pow( in vec3 v, in float e );
vec4    pow( in vec4 v, in float e );

// --------------------------------------------------------------------------------

vec3
srgb_to_linear_ref( in vec3 srgb )
{
	vec3 srgb_lo 	= srgb / SRGB_loscale;
	vec3 srgb_hi 	= pow( ( ( srgb + SRGB_hibias ) / SRGB_hiscale ), vec3( SRGB_hiexp ) );
    
	bvec3 mask 	= greaterThan( srgb, vec3( SRGB_jump ) );
	vec3 final	= mix( srgb_lo, srgb_hi, vec3( mask ) );
    
	return final;
}

vec3
linear_to_srgb_ref( in vec3 linear )
{
	vec3 linear_lo 	= linear * SRGB_loscale;
	vec3 linear_hi 	= SRGB_hiscale * pow( linear, vec3( 1. / SRGB_hiexp ) ) - SRGB_hibias;
    
	bvec3 mask 	= greaterThan( linear, vec3( SRGB_jumpinv ) );
	vec3 final	= mix( linear_lo, linear_hi, vec3( mask ) );
    
	return final;
}

vec3
srgb_to_linear_opt( in vec3 srgb )
{
    return pow( srgb, SRGB_optexp );
}

vec3
linear_to_srgb_opt( in vec3 linear )
{
    return pow( linear, SRGB_optexpinv );
}

// --------------------------------------------------------------------------------

vec3
rgb_to_xyz( in vec3 rgb )
{
	return rgb * XYZ_M;
}

vec3	
xyz_to_rgb( in vec3 xyz )
{
	return xyz * XYZ_Minv;
}

vec3
rgb_to_yiq( in vec3 rgb )
{
	return rgb * YIQ_M;
}

vec3	
yiq_to_rgb( in vec3 yiq )
{
	return yiq * YIQ_Minv;
}

// --------------------------------------------------------------------------------

vec3 
rgb_to_hsv( in vec3 rgb )
{
	vec3  hsv       = vec3( 0.0 );
	float minVal    = min_channel( rgb );
	float maxVal    = max_channel( rgb );
	float delta     = maxVal - minVal;
    
	hsv.z = maxVal;
    
	if( delta != 0.0 )
 	{
		hsv.y = delta / maxVal;
		vec3 delRGB = ( ( ( vec3(maxVal) - rgb ) / 6.0 ) + ( delta / 2.0 ) ) / delta;
        
		if (rgb.x == maxVal)        hsv.x = delRGB.z - delRGB.y;
        else if( rgb.y == maxVal )  hsv.x = ( 1.0 / 3.0 ) + delRGB.x - delRGB.z;
        else if( rgb.z == maxVal )  hsv.x = ( 2.0 / 3.0 ) + delRGB.y - delRGB.x;
        
        if( hsv.x < 0.0 )   hsv.x += 1.0; 
        if( hsv.x > 1.0 )   hsv.x -= 1.0; 
  	}
    
    return hsv;
}

vec3 
hsv_to_rgb(in vec3 hsv)
{
  	vec3 rgb = vec3(hsv.z);
    
  	if( hsv.y != 0.0 )
	{
        float var_h = hsv.x * 6.0;
    	float var_i = floor( var_h );   // Or ... var_i = floor( var_h )
    	float var_1 = hsv.z * ( 1.0 - hsv.y );
    	float var_2 = hsv.z * ( 1.0 - hsv.y * ( var_h - var_i ) );
    	float var_3 = hsv.z * ( 1.0 - hsv.y * ( 1.0 - ( var_h - var_i ) ) );
        
		if( int( var_i ) == 0 )			rgb = vec3( hsv.z, var_3, var_1 );
		else if( int( var_i ) == 1 ) 	rgb = vec3( var_2, hsv.z, var_1 );
		else if( int( var_i ) == 2 ) 	rgb = vec3( var_1, hsv.z, var_3 );
		else if( int( var_i ) == 3 ) 	rgb = vec3( var_1, var_2, hsv.z );
		else if( int( var_i ) == 4 ) 	rgb = vec3( var_3, var_1, hsv.z );		
        else                            rgb = vec3( hsv.z, var_1, var_2 );
  	}
  	return rgb;
}

// --------------------------------------------------------------------------------

float 
min_channel( in vec2 v )
{
    float t = min( v.x, v.y );
    return t;
}

float 
min_channel( in vec3 v )
{
    float t = min( v.x, min( v.y, v.z ) );
    return t;
}

float 
min_channel( in vec4 v )
{
    float t = min( v.x, min( v.y, min( v.z, v.w ) ) );
    return t;
}

float 
max_channel( in vec2 v )
{
    float t = max( v.x, v.y );
    return t;
}

float 
max_channel( in vec3 v )
{
    float t = max( v.x, max( v.y, v.z ) );
    return t;
}

float 
max_channel( in vec4 v )
{
    float t = max( v.x, max( v.y, max( v.z, v.w ) ) );
    return t;
}

// --------------------------------------------------------------------------------

float
sum( in vec2 v )
{
	return dot( v, vec2( 1. ) );
}

float
sum( in vec3 v )
{
	return dot( v, vec3( 1. ) );
}

float
sum( in vec4 v )
{
	return dot( v, vec4( 1. ) );
}

// --------------------------------------------------------------------------------

float   
mean( in vec2 v )
{
    return .5 * sum( v );
}

float   
mean( in vec3 v )
{
    return .33333333333 * sum( v );
}

float   mean( in vec4 v )
{
    return .25 * sum( v );
}

// --------------------------------------------------------------------------------

vec2
pow( in vec2 v, in float e )
{
    return pow( v, vec2( e ) );
}

vec3
pow( in vec3 v, in float e )
{
    return pow( v, vec3( e ) );
}

vec4
pow( in vec4 v, in float e )
{
    return pow( v, vec4( e ) );
}
